---
title: Stylelint Polaris
description: A configuration of Stylelint rules that promote adoption of the Polaris design system in consuming apps.
icon: WandMajor
collapseChildren: true
order: 3
keywords:
  - stylelint
  - dev tools
  - developer tools
  - tools
  - tooling
  - development
  - plugin
  - rules
  - linter
  - linting
  - css
---

[<img src="https://img.shields.io/npm/v/@shopify/stylelint-polaris.svg?labelColor=f9f9f9&color=dcf5f0" alt="npm version" style="width: 95px" />](https://www.npmjs.com/package/@shopify/stylelint-polaris/)

<picture>
  <source srcset="/images/tools/stylelint-polaris/stylelint-demo.png" media="(prefers-reduced-motion: reduce)"></source> 
  <img srcset="/images/tools/stylelint-polaris/stylelint-demo.gif" alt="Demo of Stylelint Polaris">
</picture>

## Installation

```sh
yarn add -D @shopify/stylelint-polaris stylelint
```

> Note: `stylelint-polaris` requires a peer dependency of `stylelint@>=14.15.0`

[Source code](https://github.com/Shopify/polaris/tree/main/stylelint-polaris)

## Usage

Extend `@shopify/stylelint-polaris` in your [Stylelint config](https://stylelint.io/user-guide/configure/). Example in `package.json`

```json
{
  "stylelint": {
    "extends": ["@shopify/stylelint-polaris"]
  }
}
```

> IMPORTANT: `@shopify/stylelint-polaris` must be added to the end of the `extends` array

### Run the linter

```sh
npx stylelint '**/*.{css,scss}'
```

### Run the linter and autofix failures

```sh
npx stylelint --fix '**/*.{css,scss}'
```

### Ignoring existing failures

Enabling the linter could result in a large amount of warnings and errors in existing codebases. It is important to fix as many failures upfront as possible, but that shouldn't block the linter from being added. The [styles-insert-stylelint-disable](https://github.com/Shopify/polaris/tree/main/polaris-migrator#v10) migration inserts [ignore comments](https://stylelint.io/user-guide/ignore-code/) so that enabling `stylelint-polaris` can be unblocked.

The migration will insert comments as follows:

```diff
+ // stylelint-disable-next-line -- generated by polaris-migrator DO NOT COPY
padding: 1rem;
```

Run the following command substituting `<path>` with a glob pattern of files to run against:

```sh
npx @shopify/polaris-migrator styles-insert-stylelint-disable <path>
```

## Rules

There are over 40 rules configured in Stylelint Polaris to help you avoid errors and follow stylistic and non-stylistic conventions while building for the Shopify admin. These rules help us measure the Polaris design system's coverage in the Shopify admin code base using the following categories:

- [Colors](/tools/stylelint-polaris/rules#colors)
- [Conventions](/tools/stylelint-polaris/rules#conventions)
- [Depth](/tools/stylelint-polaris/rules#depth)
- [Layout](/tools/stylelint-polaris/rules#layout)
- [Legacy](/tools/stylelint-polaris/rules#legacy)
- [Media queries](/tools/stylelint-polaris/rules#media-queries)
- [Motion](/tools/stylelint-polaris/rules#motion)
- [Shape](/tools/stylelint-polaris/rules#shape)
- [Spacing](/tools/stylelint-polaris/rules#spacing)
- [Typography](/tools/stylelint-polaris/rules#typography)
- [Z-index](/tools/stylelint-polaris/rules#z-index)

## Development

### Add new rules

1. Navigate to the root [`stylelint-polaris` config](https://github.com/Shopify/polaris/blob/main/stylelint-polaris/index.js)
1. Locate the `stylelint-polaris/coverage` options
1. Identify the appropriate category for the new rule
1. Insert the rule using standard Stylelint [rule configurations](https://stylelint.io/user-guide/configure#rules)
1. Add documentation for the rule with examples of code that will be reported as a problem and code that will fix the problem
1. The title should be the category + the stylelint rule name, for example `### colors/color-named`

```js
module.exports = {
  rules: {
    'polaris/coverage': {
      colors: {...}, // Standard Stylelint rules config
      layout: {...}, // Standard Stylelint rules config
      motion: {
        'new-rule': 'new-rule-options',
      },
    },
  },
};
```

### Build custom rules

1. Refer to the [Writing plugins](https://stylelint.io/developer-guide/plugins) guide of the Stylelint documentation
1. Create your rule in the [plugins](plugins) directory
1. Validate your plugin with [tests](https://github.com/stylelint/jest-preset-stylelint#usage) (reference sibling plugins for examples)
1. Refer to the [Add new rules](#add-new-rules) section to add your custom rule to the `stylelint-polaris` config

### Add custom messages

Custom messages are surfaced in the command line, CI, and supported editors along side the default `stylelint` rule messages. They are added to the root level config and aim to provide more insight on how to resolve rule violations.

In a majority of cases, the default rule messages are clear and concise. However, they don't always guide developers to a desired outcome. Thus, there are two mechanisms we suggest for improving and providing custom rule messages:

Set a generic custom message on the `message` property of the secondary options of a given `stylelint-polaris/coverage` category. This message is appended to the default rule message and we expect will cover most cases.

```js
module.exports = {
  rules: {
    'polaris/coverage': {
      colors: [
        {
          'color-named': 'never'
          'color-no-hex': true,
        },
        {message: 'Please use a Polaris color token: https://polaris.shopify.com/tokens/colors'},
      ],
    },
  },
}
```

Example failure message:

```diff
- Unexpected named color "red" (color-named)
+ Unexpected named color "red" (color-named) Please use a Polaris color token
```

Set a custom message on the `message` property in the [Stylelint rule config's secondary options](https://stylelint.io/user-guide/configure/#message) if supported. This message is appended to the default rule message instead of the generic category message when provided.

```js
module.exports = {
  rules: {
    'polaris/coverage': {
      layout: [
        {
          'property-disallowed-list': [
            ['position'],
            {message: 'Please use the Polaris "Sticky" component'},
          ],
        },
        {message: 'Please use a Polaris layout component'},
      ],
    },
  },
};
```

Example failure message:

```diff
- Unexpected value "sticky" for property "position" (declaration-property-value-disallowed-list) Please use a Polaris layout component
+ Unexpected value "sticky" for property "position" (declaration-property-value-disallowed-list) Please use the Polaris "Sticky" component
```

### Test `stylelint-polaris` updates in `polaris-react`

> Open your terminal to the root of the `polaris` monorepo:

1. Install and symlink dependencies

```sh
yarn install
```

2. Build `@shopify/polaris` dependencies, but not `@shopify/polaris` itself

```sh
yarn build -- --filter=@shopify/polaris^...
```

> Note: Remove the `^` character if you do want to build `@shopify/polaris`

3. Run `stylelint` in `polaris-react`

All files

```sh
yarn turbo run lint:styles --filter=@shopify/polaris
```

Specific file

```sh
yarn run stylelint path/to/component.scss

// yarn run stylelint polaris-react/src/components/TopBar/TopBar.scss
```
